package ru.dbotthepony.kstarbound.world.entities.behavior

import com.google.common.collect.ImmutableList
import org.classdump.luna.runtime.ExecutionContext
import ru.dbotthepony.kstarbound.lua.userdata.BehaviorState

class SequenceNode(val children: ImmutableList<AbstractBehaviorNode>, val isSelector: Boolean) : AbstractBehaviorNode() {
	private var index = 0

	override fun run(delta: Double, state: BehaviorState): Status {
		calls++

		while (index < children.size) {
			val child = children[index]
			val status = child.runAndReset(delta, state)

			if ((isSelector && status == Status.SUCCESS || !isSelector && status == Status.FAILURE) || status == Status.RUNNING)
				return status

			index++
		}

		return if (isSelector) Status.FAILURE else Status.SUCCESS
	}

	override fun toString(): String {
		if (isSelector)
			return "SelectorNode[$calls / ${children.size}, index=$index, current=${children.getOrNull(index)}]"
		else
			return "SequenceNode[$calls / ${children.size}, index=$index, current=${children.getOrNull(index)}]"
	}

	override fun reset() {
		children.forEach { it.reset() }
		index = 0
	}
}
